#!/usr/bin/perl

use POSIX;
use IO::Socket;
use IO::Interface;

################################################################################
#    backupDAT for networks 1.0.                                               #
#    Copyright (C) 2004 Luca Bertoncello                                       #
#    Hartigstrae 12 - 01127 Dresden Deutschland                               #
#    E-Mail: lucabert@lucabert.de                                              #
#                                                                              #
#    This program is free software; you can redistribute it and/or modify      #
#    it under the terms of the GNU General Public License as published by      #
#    the Free Software Foundation; version 2 of the License.                   #
#                                                                              #
#    This program is distributed in the hope that it will be useful,           #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of            #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
#    GNU General Public License for more details.                              #
#                                                                              #
#    You should have received a copy of the GNU General Public License         #
#    along with this program; if not, write to the Free Software               #
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
################################################################################

# Default values (you can change them by parameter):
# Device to write to
$TAPEDEV                             = '/dev/nst0';
# Set AUTOEJECT to 1 in order to eject tape at the end of backup process
# Set AUTOEJECT to 0 in order to rewind tape at the end of backup process, but not eject it
$AUTOEJECT                           = 0;
# Set VERIFYDATA to 1 in order to verify saved data
$VERIFYDATA                          = 1;

# Configurazion strings. You don't need to modify these
$MAIN_DIR                            = 'tmpBackup';
$BACKUP_FILE_PREFIX                  = 'bk';
$BACKUP_FILE_SUFFIX_GZIP             = '.tgz';
$BACKUP_FILE_SUFFIX_BZIP2            = '.tar.bz2';
$BACKUP_FILE_SUFFIX                  = '';
$TAROPTS                             = 'cfps';
$FILE_BACKUP_DIR                     = '';

# Path of the commands. You can change them, with the path in your system.
$CMDMT                               = '/bin/mt';
$CMDTAR                              = '/bin/tar';
$CMDGZIP                             = '/bin/gzip';
$CMDBZIP2                            = '/usr/bin/bzip2';

# Please DO NOT modify under this line!!
######################################################

$version                             = '0.9.9';
$port                                = 9876;
$proto                               = 'tcp';
$timeout                             = 5;
$okSignal                            = 'OK-SENDDATA';
$okCompressingSignal                 = 'OK-COMPR';
$koCompressingSignal                 = 'KO-COMPR';
$modeGzip                            = 'gzip';
$modeBzip2                           = 'bzip2';
$actionBackup                        = 'BACKUP';
$actionRestore                       = 'RESTORE';
$actionDaemon                        = 'DAEMON';
$MAXFILENAMELENGTH                   = 48;

#########################
# Main                  #
#########################
$|                                   = 1;

searchTmpDir();
getCmdLineOpts();

if($canRun)
  {
    if($action eq $actionDaemon)
      {
        if(checkUsedCmds(0) == 0)
          {
            doDaemon();
            exit(0);
          }
        else
          {
            print "Not all used commands (tar, gzip, bzip2) are available!!\n";
            exit(1);
          }
      }
    else
      {
        if(checkUsedCmds(1) == 0)
          {
            if(checkTape())
              {
                doAction();
                exit(0);
              }
            else
              {
                print "ERROR!! Tape not found!\n";
                print "Please enter tape into DAT and start the program again\n";
                exit(-2);
              }
          }
        else
          {
            print "Not all used commands (mt, tar, gzip, bzip2) are available!!\n";
            exit(1);
          }
      }
  }
exit(-1);

sub checkUsedCmds
{
  my($mode)                          = @_;

  if((! -e $CMDTAR) || (! -e $CMDGZIP) || (! -e $CMDBZIP2))
    {
      return -1;
    }
  if($mode == 1)
    {
      if(! -e $CMDMT)
        {
          return -1;
        }
    }
  return 0;
}

sub doDaemon
{
  $str = <STDIN>;
  $str =~ s/[\r\n]//g;
  execCmd($str);
}

sub doAction
{
  if($action eq $actionBackup)
    {
      doBackup();
    }
  if($action eq $actionRestore)
    {
      doRestore();
    }
}

sub doBackup
{
  my(@backupDirs, @hosts, %pwdList);

  open(DIRS, $BKLIST);
  @t                                 = <DIRS>;
  close(DIRS);
  foreach $s (@t)
    {
      $s                            =~ s/[\r\n]//g;
      if($s ne '' && !($s =~ /^#/))
        {
          push(@backupDirs, $s);
          ($h, @r)                   = split(/\:/, $s);
          push(@th, $h);
        }
    }
  @hosts                             = uniq(@th);

  print "==> Parsing config file...\n";
  $len                               = 21;
  foreach $dirH (@backupDirs)
    {
      ($host, $dir)                  = split(/\:/, $dirH);
      $len                           = (length($dir) + length($BACKUP_FILE_PREFIX)) if((length($dir) + length($prefix)) > $len);
    }
  %pwdList                           = getPasswords(@hosts);
  $ERROR                             = 0;
  # Creates the Backup
  ($a, $b, $c, $d, $m, $y, @t)       = localtime();
  $m++;
  $y                                += 1900;
  $CURDATE                           = "$d\_$m\_$y";
  $BACKUP_DIR                        = "$FILE_BACKUP_DIR/$MAIN_DIR/backup$CURDATE";
  print "==> Creating backup files...\n";
  mkdir("$FILE_BACKUP_DIR/$MAIN_DIR", 0755);
  mkdir($BACKUP_DIR, 0755);
  foreach $dirH (@backupDirs)
    {
      ($host, $dir)                  = split(/\:/, $dirH);
      $nFile                         = $dir;
      $nFile                        =~ s/\//\_/g;
      $nFile                        =~ s/\ /\-/g;
      $dir                          .= '/';
      $pathFileBackup                = "$BACKUP_DIR/$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
      $l                             = $len + 35;
      $format                        = "%-$l"."s";
      printf($format, "     Compressing [$host:$dir]...");
      if(compressDir($pathFileBackup, $host, $dir, $pwdList{$host}) == 0)
        {
          print "OK\n";
        }
      else
        {
          print "FAILED!\n";
          $ERROR                     = 1;
        }
    }
  if($ERROR != 0)
    {
      print "ERROR in compressing files!\n";
      print "Program stopped.\n";
      exit(-3);
    }
  print "==> Rewinding tape...\n";
  system("$CMDMT -f $TAPEDEV rewind");

  # Creates index file
  ($s, $min, $h, $d, $m, $y, @t)     = localtime();
  $m++;
  $y                                += 1900;
  $CURDATE                           = "$d/$m/$y";
  print "==> Creating index on tape...\n";
  $lenI                              = $len;
  $lenI                              = $MAXFILENAMELENGTH if($len > $MAXFILENAMELENGTH);
  $TMPINDEXFILE                      = "$BACKUP_DIR/index\_$d$m$y\_$h$min$s";
  open(INDEX, "> $TMPINDEXFILE");
  $NDIR                              = 0;
  print INDEX "Backup date: $CURDATE\n";
  $len1                              = $lenI + 14;
  $lenN                              = $lenI + length($BACKUP_FILE_PREFIX) + length($BACKUP_FILE_SUFFIX) - 2;
  $format                            = "| Track | %-$lenN"."s | %-$len1"."s | %-10s |\n";
  printf(INDEX $format, "File", "Remarks", "Length");
  print INDEX "+-------+";
  for($l = 0; $l < ($lenI + length($BACKUP_FILE_PREFIX) + length($BACKUP_FILE_SUFFIX)); $l++)
    {
      print INDEX "-";
    }
  print INDEX "+";
  for($l = 0; $l < ($lenI + 16); $l++)
    {
      print INDEX "-";
    }
  print INDEX "+------------+\n";

  $TOTAL                             = 0;
  $len1                              = $lenI + 14;
  $lenN                              = $lenI + length($BACKUP_FILE_PREFIX) + length($BACKUP_FILE_SUFFIX) - 2;
  $format                            = "| %5d | %-$lenN"."s | %-$len1"."s | %10s |\n";
  printf(INDEX $format, $nDir, "index.txt", "This file", "");
  $NDIR++;
  $fileLen                           = fileLength('backupDAT.pl');
  $TOTAL                            += $fileLen;
  printf(INDEX $format, $NDIR, "backupDAT.pl", "Backup/Restore script", formatFileLength($fileLen));
  $NDIR++;
  foreach $dirH (@backupDirs)
    {
      ($host, $dir)                  = split(/\:/, $dirH);
      $nFile                         = $dir;
      $nFile                        =~ s/\//\_/g;
      $nFile                        =~ s/\ /\-/g;
      $dir                          .= '/';
      $pathFileBackup                = "$BACKUP_DIR/$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
      $file                          = "$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
      $file                          =~ s/\//\_/g;
      $fileLen                       = fileLength($pathFileBackup);
      $TOTAL                        += $fileLen;
      printf(INDEX $format, $NDIR, $file, "Backup $host:$dir", formatFileLength($fileLen)) if(length($file) < $MAXFILENAMELENGTH);
      $nR                            = 0;
      while(length($file) > $MAXFILENAMELENGTH)
        {
          $format1                   = "|       | %-$lenN"."s | %-$len1"."s |            |\n";
          $f1                        = substr($file, 0, $MAXFILENAMELENGTH - 1)."\\";
          $d1                        = substr($dir, 0, $MAXFILENAMELENGTH - 1)."\\";
          if($nR == 0)
            {
              printf(INDEX $format, $NDIR, $f1, "Backup $host:$d1", formatFileLength($fileLen));
            }
          else
            {
              printf(INDEX $format1, $f1, $d1);
            }
          $file                      = '  '.substr($file, $MAXFILENAMELENGTH - 1);
          $dir                       = '  '.substr($dir, $MAXFILENAMELENGTH - 1);
          $nR++;
        }
      if($nR != 0)
        {
          printf(INDEX $format1, $file, $dir);
        }
      $NDIR++;
    }
  print INDEX "+-------+";
  for($l = 0; $l < ($lenI + length($BACKUP_FILE_PREFIX) + length($BACKUP_FILE_SUFFIX)); $l++)
    {
      print INDEX "-";
    }
  print INDEX "+";
  for($l = 0; $l < ($lenI + 16); $l++)
    {
      print INDEX "-";
    }
  print INDEX "+------------+\n";
  print INDEX "Total: ".formatFileLength($TOTAL)."\n";
  close(INDEX);
  system("$CMDTAR cf $TAPEDEV $TMPINDEXFILE 2> /dev/null");

  print "==> Storing backup/restore script on tape...\n";
  system("$CMDTAR cf $TAPEDEV $0 2> /dev/null");

  # Storing the Backup
  print "==> Storing backup on tape...\n";
  foreach $dirH (@backupDirs)
    {
      ($host, $dir)                  = split(/\:/, $dirH);
      $nFile                         = $dir;
      $nFile                        =~ s/\//\_/g;
      $nFile                        =~ s/\ /\-/g;
      $dir                          .= '/';
      $pathFileBackup                = "$BACKUP_DIR/$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
      $file                          = "$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
      $file                          =~ s/\//\_/g;
      $lenn                          = $len + 35;
      $format                        = "%-$lenn"."s";
      printf($format, "     Storing [$host:$dir] on tape...");
      $ret                          = system("$CMDTAR cf $TAPEDEV $pathFileBackup 2> /dev/null");
      if($ret == 0)
        {
          print "OK\n";
        }
      else
        {
          print "FAILED!\n";
          $ERROR                     = 1;
        }
    }
  if($ERROR != 0)
    {
      print "ERROR in storing files!\n";
      print "Program stopped.\n";
      exit(-3);
    }

  $VERIFY                            = 0;
  if($VERIFYDATA == 1)
    {
      print "==> Rewinding tape...\n";
      system("$CMDMT -f $TAPEDEV rewind");
      sleep(1);

      # Verifying the Backup
      print "==> Verifying backup...\n";
      system("$CMDMT -f $TAPEDEV fsf 2");    # Jump over index and restore script
      sleep(1);
      foreach $dirH (@backupDirs)
        {
          ($host, $dir)              = split(/\:/, $dirH);
          $nFile                     = $dir;
          $nFile                    =~ s/\//\_/g;
          $nFile                    =~ s/\ /\-/g;
          $dir                      .= '/';
          $pathFileBackup            = "$BACKUP_DIR/$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
          $file                      = "$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
          $file                     =~ s/\//\_/g;
          $lenn                      = $len + 35;
          $format                    = "%-$lenn"."s";
          printf($format, "     Verifying [$host:$dir]...");
          $ret                       = system("$CMDTAR tf $TAPEDEV > /dev/null 2> /dev/null");
          if($ret == 0)
            {
              print "OK\n";
            }
          else
            {
              print "FAILED!\n";
              $ERROR                 = 1;
            }
          sleep(1);
          system("$CMDMT -f $TAPEDEV fsf 1");
        }
    }

  print "==> Removing temp files...\n";
  unlink($TMPINDEXFILE);
  foreach $dirH (@backupDirs)
    {
      ($host, $dir)                  = split(/\:/, $dirH);
      $nFile                         = $dir;
      $nFile                        =~ s/\//\_/g;
      $nFile                        =~ s/\ /\-/g;
      $pathFileBackup                = "$BACKUP_DIR/$BACKUP_FILE_PREFIX.$host$nFile$BACKUP_FILE_SUFFIX";
      unlink($pathFileBackup);
    }
  rmdir($BACKUP_DIR);
  rmdir("$FILE_BACKUP_DIR/$MAIN_DIR");

  endOfProcedure();

  if($VERIFY == 0)
    {
      print "==> Backup completed successfully!\n";
      print "Saved $TOTALSPACE in ".($NDIR - 2)." file(s)\n";
    }
  else
    {
      print "==> ERRORS IN BACKUP!!\n";
      exit(-1);
    }
}

sub fileLength
{
  my($file)                          = @_;

  ($dev, $ino, $mode, $nlink, $uid,
   $gid, $rdev, $size, $atime, $mtime,
   $ctime, $blksize, $blocks)        = stat($file);

  return($size);
}

sub compressDir
{
  my($file, $host, $dir, $pwd)       = @_;

  return sendCmd($host, $file, $dir, $pwd, $mode, 0);
}

sub doRestore
{
  print "==> Rewinding tape...\n";
  system("$CMDMT -f $TAPEDEV rewind");

  print "==> Reading index from tape...\n";
  mkdir("$FILE_BACKUP_DIR/$MAIN_DIR", 0755);
  ($s, $min, $h, $d, $m, $y, @t)     = localtime();
  $m++;
  $y                                += 1900;
  $CURDATE                           = "$d/$m/$y";
  $TMPINDEXFILE                      = "$FILE_BACKUP_DIR/$MAIN_DIR/index\_$d$m$y\_$h$min$s";
  system("$CMDTAR xfO $TAPEDEV > $TMPINDEXFILE");
  open(INDEX, $TMPINDEXFILE);
  @indexFile                         = <INDEX>;
  close(INDEX);
  %files                             = ();
  foreach $file (@indexFile)
    {
      $file                          =~ s/[\r\n]//g;
      @fields                        = split(/\|/, $file);
      $fields[1]                     =~ s/\ //g;
      $fields[2]                     =~ s/\ //g;
      $index                         = '';
      $name                          = '';
      $index                         = $fields[1] if(($file =~ /^\|/) && (!($file =~ /Track/)) && ($fields[1] > 1));
      $name                          = $fields[2] if(($file =~ /^\|/) && (!($file =~ /Track/)) && ($fields[1] > 1));
      if($index ne '')
        {
          $files{$index}             = $name;
        }
    }

  displayList(@indexFile);

  $quit                              = 0;
  while($quit == 0)
    {
      print "=> ";
      $cmd                           = <STDIN>;
      $cmd                           =~ s/[\r\n]//g;
      $recognized                    = 0;
      if(!$recognized && ($cmd eq 'list'))
        {
          displayList(@indexFile);
          $recognized                = 1;
        }
      if(!$recognized && ($cmd eq 'help'))
        {
          displayCommandPrompt();
          $recognized                = 1;
        }
      if(!$recognized && ($cmd eq 'quit'))
        {
          $quit                      = 1;
          $recognized                = 1;
        }
      if(!$recognized && ($cmd eq 'extract all'))
        {
          $recognized                = 1;
          doExtract(%files);
        }
      if(!$recognized && (substr($cmd, 0, 8) eq 'extract '))
        {
          $track                     = substr($cmd, 8);
          $recognized                = 1;
          if($track eq "0" || $track eq "1")
            {
              print "Tracks 0 and 1 are reserved (index file and restore script)!\n";
              print "You don't need to extract them.\n";
            }
          else
            {
              $ok                    = 0;
              %eTrack                = ();
              foreach $key (sort keys(%files))
                {
                  $pat               = $track;
                  $pat               =~ s/\?/\(\[\\w\\d\\\.\]\)/g;
                  $pat               =~ s/\*/\(\[\\w\\d\\\.\]\)\*/g;
                  if(($files{$key} =~ /^$pat$/) || ($key eq $track))
                    {
                      $ok            = 1;
                      $eTrack{$key}  = $files{$key};
                    }
                }
              if($ok == 1)
                {
                  doExtract(%eTrack);
                }
              else
                {
                  print "Invalid file/track!\n";
                }
            }
        }
      print "Unknown command!\n" if($recognized == 0);
    }

  unlink($TMPINDEXFILE);
  rmdir("$FILE_BACKUP_DIR/$MAIN_DIR");

  endOfProcedure();
}

sub doExtract
{
  my(%tracks)                        = @_;
  my($ok, $len, $ERRORS, $POS, $firstFile, $PATHTOCOPY);

  $ok                                = 0;
  while($ok == 0)
    {
      print "Enter path to extract file(s) to: ";
      $PATHTOCOPY                    = <STDIN>;
      $PATHTOCOPY                    =~ s/[\r\n]//g;
      if($PATHTOCOPY eq '' || !(-d $PATHTOCOPY))
        {
          print "Invalid path!\n";
        }
      else
        {
          $ok                        = 1;
        }
    }

  $len                               = 0;
  foreach $key (sort keys(%tracks))
    {
      $firstFile                     = $key if($len == 0);
      $len                           = length($tracks{$key}) if(length($tracks{$key}) > $len);
    }
  $len                              += 35;
  $format                            = "%-$len"."s";
  $ERRORS                            = 0;
  print "==> Positioning tape...\n";
  # Jump to first file in the list
  system("$CMDMT -f $TAPEDEV asf $firstFile");
  $POS                               = $firstFile;
  print "==> Extracting file(s)...\n";
  foreach $key (sort keys(%tracks))
    {
      printf($format, "     Extracting '$tracks{$key}'...");
      $ret                           = system("$CMDTAR xOf $TAPEDEV > $PATHTOCOPY/$tracks{$key} 2> /dev/null");
      if($ret == 0)
        {
          print "OK\n";
        }
      else
        {
          print "FAILED!\n";
          $ERROR                     = 1;
        }
      system("$CMDMT -f $TAPEDEV fsf 1");
      $POS++;
    }
  if($ERROR == 0)
    {
      print "==> Extract completed successfully!\n";
    }
  else
    {
      print "==> ERRORS IN EXTRATING FILE(S)!!\n";
      exit(-3);
    }
}

sub displayList
{
  my(@indexFile)                     = @_;

  print "\n";
  foreach $line (@indexFile)
    {
      print "$line\n";
    }
  displayCommandPrompt();
}

sub displayCommandPrompt
{
  print "\n";
  print "Available functions:\n";
  print " - extract <nFile|fileName>      extract file with number <nFile>\n";
  print "                                 or with name <fileName>.\n";
  print "                                 (? and * are allowed).\n";
  print " - extract all                   extract all files on tape.\n";
  print " - help                          display this help.\n";
  print " - list                          display list of files on tape.\n";
  print " - quit                          exit.\n";
}

sub getCmdLineOpts
{
  $canRun                            = 1;
  $action                            = $actionBackup;
  $mode                              = $modeGzip;
  for($nA = 0; $nA <= $#ARGV; $nA++)
    {
      $recognized                    = 0;
      if(!$recognized && $ARGV[$nA] eq 'backup')
        {
          $recognized                = 1;
          $action                    = $actionBackup;
        }
      if(!$recognized && $ARGV[$nA] eq 'restore')
        {
          $recognized                = 1;
          $action                    = $actionRestore;
        }
      if(!$recognized && $ARGV[$nA] eq 'daemon')
        {
          $recognized                = 1;
          $action                    = $actionDaemon;
        }
      if(!$recognized && ($ARGV[$nA] eq '-l' || $ARGV[$nA] eq '--bklist'))
        {
          $BKLIST                    = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($BKLIST eq '' || !(-e $BKLIST))
            {
              print "Unspecified or invalid backup list!\n";
              $canRun                = 0;
            }
        }
      if(!$recognized && ($ARGV[$nA] eq '-d' || $ARGV[$nA] eq '--dev'))
        {
          $TAPEDEV                   = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($TAPEDEV eq '')
            {
              print "Unspecified device!\n";
              $canRun                = 0;
            }
        }
      if(!$recognized && ($ARGV[$nA] eq '-e' || $ARGV[$nA] eq '--eject'))
        {
          $str                       = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($str eq '')
            {
              print "Eject: Unspecified value!\n";
              $canRun                = 0;
            }
          else
            {
              if($str eq 'true')
                {
                  $AUTOEJECT         = 1;
                }
              else
                {
                  if($str eq 'false')
                    {
                      $AUTOEJECT     = 0;
                    }
                  else
                    {
                      print "Eject: Invalid value!\n";
                      $canRun        = 0;
                    }
                }
            }
        }
      if(!$recognized && ($ARGV[$nA] eq '-f' || $ARGV[$nA] eq '--verify'))
        {
          $str                       = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($str eq '')
            {
              print "Verify: Unspecified value!\n";
              $canRun                = 0;
            }
          else
            {
              if($str eq 'true')
                {
                  $VERIFYDATA        = 1;
                }
              else
                {
                  if($str eq 'false')
                    {
                      $VERIFYDATA    = 0;
                    }
                  else
                    {
                      print "Verify: Invalid value!\n";
                      $canRun        = 0;
                    }
                }
            }
        }
      if(!$recognized && ($ARGV[$nA] eq '-t' || $ARGV[$nA] eq '--tmpdir'))
        {
          $FILE_BACKUP_DIR           = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($FILE_BACKUP_DIR eq '' || !(-e $FILE_BACKUP_DIR))
            {
              print "Invalid temporary directory!\n";
              $canRun                = 0;
            }
        }
      if(!$recognized && ($ARGV[$nA] eq '-s' || $ARGV[$nA] eq '--email'))
        {
          $EMAIL                     = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($EMAIL eq '')
            {
              print "Unspecified E-Mail!\n";
              $canRun                = 0;
            }
        }
      if(!$recognized && $ARGV[$nA] eq '--with-gzip')
        {
          $CMDGZIP                   = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($CMDGZIP eq '')
            {
              print "Unspecified path!\n";
              $canRun                = 0;
            }
        }
      if(!$recognized && $ARGV[$nA] eq '--with-bzip2')
        {
          $CMDBZIP2                  = $ARGV[$nA + 1];
          $nA++;
          $recognized                = 1;
          if($CMDBZIP2 eq '')
            {
              print "Unspecified path!\n";
              $canRun                = 0;
            }
        }
      if(!$recognized && ($ARGV[$nA] eq '-g' || $ARGV[$nA] eq '--gzip'))
        {
          $recognized                = 1;
          $mode                      = $modeGzip;
          $BACKUP_FILE_SUFFIX        = $BACKUP_FILE_SUFFIX_GZIP;
        }
      if(!$recognized && ($ARGV[$nA] eq '-j' || $ARGV[$nA] eq '--bzip2'))
        {
          $recognized                = 1;
          $mode                      = $modeBzip2;
          $BACKUP_FILE_SUFFIX        = $BACKUP_FILE_SUFFIX_BZIP2;
        }
      if(!$recognized && ($ARGV[$nA] eq '-v' || $ARGV[$nA] eq '--version'))
        {
          version();
          $canRun                    = 0;
          $recognized                = 1;
        }
      if(!$recognized && ($ARGV[$nA] eq '-h' || $ARGV[$nA] eq '--help'))
        {
          help();
          $canRun                    = 0;
          $recognized                = 1;
        }
      if($recognized != 1)
        {
          $canRun                    = 0;
          print "Unrecognized option [$ARGV[$nA]]\n\n";
          help();
        }
    }
}

# Searches for temp directory
sub searchTmpDir
{
  @tmpDirList=('/var/tmp', '/var/temp', '/usr/tmp', '/usr/temp', '/tmp');

  foreach $dir (@tmpDirList)
    {
      $FILE_BACKUP_DIR               = $dir if(-e $dir);
    }
}

# Checks for Tape
sub checkTape
{
  $str                               = `$CMDMT -f $TAPEDEV stat`;
  if($str =~ /ONLINE/)
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

# Exit from program, rewinding or ejecting tape (see configurazion/parameter)
sub endOfProcedure
{
  sleep(1);
  if($AUTOEJECT == 1)
    {
      print "==> Ejecting tape...\n";
      system("$CMDMT -f $TAPEDEV offline");
    }
  else
    {
      print "==> Rewinding tape...\n";
      system("$CMDMT -f $TAPEDEV rewind");
    }
}

sub version
{
  print "BackupDAT for networks ver. $version\n";
  print "Script for backup and restore on tape. Network version.\n";
  print "By Luca Bertoncello <lucabert\@lucabert.de>\n";
  print "The program was released under GPL v. 2.0 license - NO WARRANTY PROVIDED -\n";
  print "For information about the program and the author:\n";
  print "\t\thttp://www.lucabert.de/\n";
}

sub help
{
  version();
  if($AUTOEJECT)
    {
      $AUTOEJECTSTR                  = 'true';
    }
  else
    {
      $AUTOEJECTSTR                  = 'false';
    }
  if($VERIFYDATA)
    {
      $VERIFYDATASTR                 = 'true';
    }
  else
    {
      $VERIFYDATASTR                 = 'false';
    }
  print "\nUsage:\n";
  print "backupDAT.pl [backup|restore|daemon] <parameters>\n\n";
  print "backup                       Make a backup (default)\n";
  print "restore                      Make a restore\n";
  print "daemon                       Run as daemon (on all servers to be backupped)\n";
  print "Parameters:\n";
  print "  -v --version               Prints version of programm\n";
  print "  -h --help                  Display this help\n";
  print "  -l --bklist <file>         Use <file> for backup list.\n";
  print "                             If not specified, it will be asked.\n";
  print "  -d --dev <dev>             Use <dev> as device to write to\n";
  print "                             (default: $TAPEDEV).\n";
  print "  -e --eject <true|false>    Eject tape at the end of backup/restore\n";
  print "                             process (default: $AUTOEJECTSTR).\n";
  print "  -f --verify <true|false>   Verify saved files at the end of backup\n";
  print "                             process (default: $VERIFYDATASTR).\n";
  print "  -t --tmpdir <dir>          Use <dir> as temporary directory.\n";
  print "                             (Default: $FILE_BACKUP_DIR).\n";
  print "  -s --email <address>       Send the result of backup to the\n";
  print "                             given E-Mail address.\n";
  print "  -g --gzip                  Use gzip als compression program (default)\n";
  print "  -j --bzip2                 Use bzip2 als compression program\n";
  print "  --with-gzip <path>         Use this path for gzip (default: $CMDGZIP)\n";
  print "  --with-bzip2 <path>        Use this path for bzip2 (default: $CMDBZIP2)\n\n";
  print "Starting the program without parameters cause starting backup\n";
}

sub execCmd
{
  my($cmd, $localFile)               = @_;

  ($rec, $iPwd, $dir, $mode)         = split(/\:/, $cmd);
  if($rec ne 'BKG')
    {
      print "ILLEGAL COMMAND\n";
      return(-3);
    }
  else
    {
      if(defined($localFile))
        {
          print "Local compressing [$dir]. Mode: [$mode]\n" if($verbose != 0);
          if($mode eq $modeGzip)
            {
              $CMDCOMPRESS = $CMDGZIP;
            }
          else
            {
              $CMDCOMPRESS = $CMDBZIP2;
            }
          $tmpFile     = generateTempFile();
          $cmdTar      = "$CMDTAR $TAROPTS $tmpFile \"$dir\" 2> /dev/null";
          $cmdCompr    = "$CMDCOMPRESS - > $localFile < $tmpFile 2> /dev/null";
          $ret         = system($cmdTar);
          if($ret == 0)
            {
              $ret     = system($cmdCompr);
              if($ret == 0)
                {
                  print "SUCCESS!\n" if($verbose != 0);
                }
              else
                {
                  print "Problem by compressing directory...\n";
                  unlink($tmpFile);
                  return(-1);
                }
            }
          else
            {
              print "Problem by compressing directory...\n";
              unlink($tmpFile);
              return(-1);
            }
          unlink($tmpFile);
        }
      else
        {
          ($username, $pwd ) = (getpwnam "root");
          if(crypt($iPwd, $pwd) ne $pwd)
            {
              print "INVALID PASSWORD\n";
              return(-2);
            }
          else
            {
              print "$okSignal\n";
              if($mode eq $modeGzip)
                {
                  $CMDCOMPRESS = $CMDGZIP;
                }
              else
                {
                  $CMDCOMPRESS = $CMDBZIP2;
                }
              $tmpFile     = generateTempFile();
              $tmpFileC    = generateTempFile();
              $cmdTar      = "$CMDTAR $TAROPTS $tmpFile \"$dir\" 2> /dev/null";
              $cmdCompr    = "$CMDCOMPRESS - > $tmpFileC < $tmpFile 2> /dev/null";
              $ret         = system($cmdTar);
              if($ret == 0)
                {
                  $ret     = system($cmdCompr);
                  if($ret == 0)
                    {
                      print "$okCompressingSignal\n";
                    }
                  else
                    {
                      print "$koCompressingSignal\n";
                      unlink($tmpFile);
                      unlink($tmpFileC);
                    }
                }
              else
                {
                  print "$koCompressingSignal\n";
                  unlink($tmpFile);
                  unlink($tmpFileC);
                }
              open(DATA, $tmpFileC);
              while(($s = <DATA>))
                {
                  print $s;
                }
              close(DATA);
              unlink($tmpFile);
              unlink($tmpFileC);
            }
        }
    }
  return(0);
}

sub sendCmd
{
  my($server, $localFile, $remoteDir, $pwd, $mode, $verbose) = @_;

  if(isLocalIP($server))
    {
      print "Execute local\n" if($verbose != 0);
      return execCmd("BKG::$remoteDir:$mode", $localFile);
    }
  else
    {
      print "Send per Net...\n" if($verbose != 0);
      return sendCmdNet($server, $pwd, $localFile, $remoteDir, $mode);
    }
}

sub sendCmdNet
{
  my($server, $pwd, $localFile, $remoteDir, $mode) = @_;

  $ipPack                            = gethostbyname($server);;
  ($a, $b, $c, $d)                   = unpack('C4', $ipPack);
  $ip                                = "$a.$b.$c.$d";
  $socket                            = IO::Socket::INET->new(PeerAddr => $server,
                                                             PeerPort => $port,
                                                             Proto    => $proto,
                                                             Timeout  => $timeout);
  if($socket)
    {
      $socket->print("BKG:$pwd:$remoteDir:$mode\r\n");
      $ret                           = $socket->getline();
      $ret                           =~ s/[\r\n]//g;
      if($ret eq $okSignal)
        {
          print "Got answer! Waiting for data...\n" if($verbose != 0);
          $ret                       = $socket->getline();
          $ret                       =~ s/[\r\n]//g;
          if($ret eq $okCompressingSignal)
            {
              print "Saving data to [$localFile]...\n" if($verbose != 0);
              open(OUTFILE, "> $localFile");
              while(defined($buf = <$socket>))
                {
                  print OUTFILE $buf;
                }
              close(OUTFILE);
            }
          else
            {
              print "Problem by compressing directory...\n";
              return(-1);
            }
        }
      else
        {
          print "ERROR: [$ret]\n";
          return(-1);
        }
      return(0);
    }

  print STDERR "Unable to contact remote host [$server]\n";
  return(-2);
}

sub isLocalIP
{
  my($host)                          = @_;
  my($local, $ipPack, $ip, $sock, $eth, @eths, $ipEth);

  $ipPack                            = gethostbyname($host);;
  ($a, $b, $c, $d)                   = unpack('C4', $ipPack);
  $ip                                = "$a.$b.$c.$d";
  $sock                              = IO::Socket::INET->new(Proto => 'udp');
  @eths                              = $sock->if_list();
  $local                             = 0;
  foreach $eth (@eths)
    {
      $ipEth                         = $sock->if_addr($eth);
      if($ipEth eq $ip)
        {
          $local                     = 1;
        }
    }
  $sock->close();

  return $local;
}

sub uniq
{
  my(@list)                          = @_;
  my(%seen, $item, @uniq);

  %seen = ();
  foreach $item (@list)
    {
      push(@uniq, $item) unless $seen{$item}++;
    }

  return(@uniq);
}

sub getPasswords
{
  my(@hosts)                         = @_;
  my(%passwd);

  %passwd                            = ();
  foreach $h (@hosts)
    {
      if(!isLocalIP($h))
        {
          print "Enter root password for [$h]: ";
          system("stty -echo");
          $pwd                       = <STDIN>;
          $pwd                      =~ s/[\r\n]//g;
          print "\n";
          system("stty echo");
          $passwd{$h}                = $pwd;
        }
      else
        {
          $passwd{$h}                = 'LOCAL';
        }
    }

  return(%passwd);
}

sub generateTempFile
{
  $file = $FILE_BACKUP_DIR.'/backup.'.time();
  $n = 1;
  $fileN = $file;
  while(-e $fileN)
    {
      $fileN = $file.".$n";
      $n++;
    }
  open(F, ">$fileN");
  close(F);
  return $fileN;
}

sub formatFileLength
{
  my($length)                        = @_;
  my @units                          = ('B', 'KB', 'MB', 'GB', 'TB');
  my($index);

  $index                             = 0;
  while($length > 1024)
    {
      $length                        = $length / 1024;
      $index++;
    }

  return(sprintf("%4.2f %s", $length, $units[$index]));
}
